# after it is instantiated each class will load a corrseponding data set and have the following public variables
# train_texts, dev_texts, test_texts, train_labels, dev_labels, test_labels
import pandas as pd
import json
import numpy as np

class TRACAggressionReader:
    def __init__(self, folder_path):
        print("Loading the TRAC Agressive speech on Facebook data ...")
        df_train = pd.read_csv(folder_path + '/english/agr_en_train.csv', header = None, encoding = "utf-8") 
        df_dev = pd.read_csv(folder_path + '/english/agr_en_dev.csv', header = None, encoding = "utf-8") 
        df_test = pd.read_csv(folder_path + '/labeled_test_data/agr_en_fb_gold.csv', header = None, encoding = "utf-8") 
        
        self.train_texts = df_train[1]
        self.dev_texts = df_dev[1]
        self.test_texts = df_test[1]
        

        self.train_labels = df_train[2]
        self.dev_labels = df_dev[2]
        self.test_labels = df_test[2]
        
        mapping_lambda = lambda x : 0 if x == "NAG" else 1

        self.train_labels = self.train_labels.apply(mapping_lambda)
        self.dev_labels = self.dev_labels.apply(mapping_lambda)
        self.test_labels = self.test_labels.apply(mapping_lambda)
        print("Finished loading!")

class GaoReader:
    def __init__(self, folder_path, append_title = "False"):
        print ("Loading the Gao et al. dataset ...")
        title = {}
        # for now we wont use titles but i'm keeping this so we can add them later if we want
        title["30"] = ""
        title["31"] = ""
        title["32"] = ""
        title["33"] = ""
        title["35"] = ""
        title["40"] = ""
        title["41"] = ""
        title["42"] = ""
        title["43"] = ""
        title["44"] = ""

        self.train_texts = []
        self.dev_texts = []
        self.test_texts = []

        self.train_labels = []
        self.dev_labels = []
        self.test_labels = []
        
        joint_texts = []
        joint_labels = []
        keys = sorted(title.keys())
        for file_prefix in keys:
            for line in open(folder_path + "/fox-news-comments-master/annotated-threads/" + file_prefix + "-comments.txt"):
                elems = line.split(" ")
                label = 1 if elems[0].startswith("1") else 0
                text = " ".join(elems[1:]).replace("\n"," ")
                
                #if(file_prefix == "41" or file_prefix == "42"):
                #    self.dev_texts.append(text)
                #    self.dev_labels.append(label)
                #elif(file_prefix == "43" or file_prefix == "44"):
                #    self.test_texts.append(text)
                #    self.test_labels.append(label)
                #else:
                #    self.train_texts.append(text)
                #    self.train_labels.append(label)
                joint_texts.append(text)
                joint_labels.append(label)

        indexes = pd.Series(range(len(joint_labels)))
        indexes = np.array(indexes.sample(len(joint_labels), random_state = 42)) # this shuffles the indexes
        indexes = np.array(indexes)
        dev_starts = int(0.7*len(indexes))
        test_starts = int(0.8*len(indexes)) 
        train_indexes = indexes[:dev_starts]
        dev_indexes = indexes[dev_starts:test_starts]
        test_indexes = indexes[test_starts:]

        for ind in train_indexes:
            self.train_texts.append(joint_texts[ind])
            self.train_labels.append(joint_labels[ind])
        for ind in dev_indexes:
            self.dev_texts.append(joint_texts[ind])
            self.dev_labels.append(joint_labels[ind])
        for ind in test_indexes:
            self.test_texts.append(joint_texts[ind])
            self.test_labels.append(joint_labels[ind])
        self.train_texts = pd.Series(self.train_texts)
        self.dev_texts = pd.Series(self.dev_texts)
        self.test_texts = pd.Series(self.test_texts)

        self.train_labels = pd.Series(self.train_labels)
        self.dev_labels = pd.Series(self.dev_labels)
        self.test_labels = pd.Series(self.test_labels)
        
        print("Finished loading!")


class KaggleToxicJigsawReader:
    def __init__(self, folder_path):
        print("Loading the Kaggle Toxic Jigsaw data ...")
        
        # TRAIN AND DEV PARTS
        df_train = pd.read_csv(folder_path + '/all/train.csv', encoding = "utf-8") 
        
        df_train['comment_text'] = df_train['comment_text'].apply(lambda s: s.replace("\n", " "))
        df_train['final_label'] = (df_train['toxic'] + df_train['severe_toxic'] + df_train['obscene'] + 
                                   df_train['threat'] + df_train['insult'] + df_train['identity_hate']).apply(lambda x: min(x,1))
        
        df_train_train = df_train.sample(frac = 0.9, random_state = 42)
        df_train_dev = df_train.drop(df_train_train.index)
        
        self.train_texts = df_train_train['comment_text']
        self.dev_texts = df_train_dev['comment_text']
        
        self.train_labels = df_train_train['final_label']
        self.dev_labels = df_train_dev['final_label']
        
        # TEST SET IS A BIT DIFFERENT
        df_test = pd.read_csv(folder_path + '/all/test.csv', encoding = "utf-8") 
        df_test_labels = pd.read_csv(folder_path + '/all/test_labels.csv', encoding = "utf-8") 
        
        df_test['comment_text'] = df_test['comment_text'].apply(lambda s: s.replace("\n", " "))
        df_test['final_label'] = (df_test_labels['toxic'] + df_test_labels['severe_toxic'] + df_test_labels['obscene'] + 
                                   df_test_labels['threat'] + df_test_labels['insult'] + df_test_labels['identity_hate']).apply(lambda x: min(x,1))
        df_test = df_test[df_test['final_label'] >= 0]
        self.test_texts = df_test['comment_text']
        self.test_labels = df_test['final_label']

        print("Finished loading!")



class KolhatkarReader:
    def __init__(self, folder_path):
        print("Loading the Kolhatkar data ...")
        df = pd.read_csv(folder_path + '/SFU-SFU-_SOCC/SOCC/annotated/constructiveness/SFU_constructiveness_toxicity_corpus.csv', encoding = "utf-8") 
        
        # the list comprehension turns 1-4 ratings to 0-3. max takes the biggest of them, and the min part converts 0 to 0 and 1-3 to 1 
        # this boils down to comments that have at least one toxicity rathing >1 in the file to be labeled as 1 and all others as 0
        df["final_label"] = df["toxicity_level"].apply(lambda s: min(max( [int(x)-1 for x in s.split("\n")]),1))
        # split the data into train (80%) and test (20%)
        df_train = df.sample(frac = 0.8, random_state = 42)
        df_test = df.drop(df_train.index)
 
        # further split the train into a train and dev (7/8 of the 80% is 70% of the 
        # total data for training which
        # leaves 10% of total data for the dev set)
        df_train_train = df_train.sample(frac = 7.0/8.0, random_state = 9001)
        df_train_dev = df_train.drop(df_train_train.index)
 
       
        self.train_texts = df_train_train['comment_text']
        self.dev_texts = df_train_dev['comment_text']
        self.test_texts = df_test['comment_text']
        

        self.train_labels = df_train_train['final_label']
        self.dev_labels = df_train_dev['final_label']
        self.test_labels = df_test['final_label']
         
        print("Finished loading!")


class WaseemNAACLReader:
    def __init__(self, folder_path):
        print("Loading the WaseemNAACL data ...")
        
        texts = []
        labels = []
        
        for line in open(folder_path + "/neither.json", encoding = "utf-8"):
            data = json.loads(line)
            text = data["text"]
            texts.append(text)
            labels.append(0)

        for line in open(folder_path + "/sexism.json", encoding = "utf-8"):
            data = json.loads(line)
            text = data["text"]
            texts.append(text)
            labels.append(1)

        for line in open(folder_path + "/racism.json", encoding = "utf-8"):
            data = json.loads(line)
            text = data["text"]
            texts.append(text)
            labels.append(1)

        df = pd.DataFrame()
        df["comment_text"] = texts
        df["final_label"] = labels
        df = df.sample(frac=1, random_state = 2).reset_index(drop=True) # shuffle the rows 
        
        # split the data into train (80%) and test (20%)
        df_train = df.sample(frac = 0.8, random_state = 42)
        df_test = df.drop(df_train.index)
 
        # further split the train into a train and dev (7/8 of the 80% is 70% of the 
        # total data for training which
        # leaves 10% of total data for the dev set)
        df_train_train = df_train.sample(frac = 7.0/8.0, random_state = 9001)
        df_train_dev = df_train.drop(df_train_train.index)
 
       
        self.train_texts = df_train_train['comment_text']
        self.dev_texts = df_train_dev['comment_text']
        self.test_texts = df_test['comment_text']
        

        self.train_labels = df_train_train['final_label']
        self.dev_labels = df_train_dev['final_label']
        self.test_labels = df_test['final_label']

        print("Finished loading!")


class WaseemNLPReader:
    def __init__(self, folder_path):
        print("Loading the WaseemNLP data ...")
        
        texts = []
        labels = []
        
        for line in open(folder_path + "/amateur_expert.json", encoding = "utf-8"):
            data = json.loads(line)
            text = data["text"]
            texts.append(text)
            label = 0 if data["Annotation"] == "Neither" else 1
            labels.append(label)

        df = pd.DataFrame()
        df["comment_text"] = texts
        df["final_label"] = labels
        df = df.sample(frac=1, random_state = 2).reset_index(drop=True) # shuffle the rows 
        
       # split the data into train (80%) and test (20%)
        df_train = df.sample(frac = 0.8, random_state = 42)
        df_test = df.drop(df_train.index)
 
        # further split the train into a train and dev (7/8 of the 80% is 70% of the 
        # total data for training which
        # leaves 10% of total data for the dev set)
        df_train_train = df_train.sample(frac = 7.0/8.0, random_state = 9001)
        df_train_dev = df_train.drop(df_train_train.index)
 
       
        self.train_texts = df_train_train['comment_text']
        self.dev_texts = df_train_dev['comment_text']
        self.test_texts = df_test['comment_text']
        

        self.train_labels = df_train_train['final_label']
        self.dev_labels = df_train_dev['final_label']
        self.test_labels = df_test['final_label']

        print("Finished loading!")


def WulczynAnnotationMap(x):
    return 1 if x >= 0.5 else 0

class WulczynReader():
    def __init__(self, comments_filename, annotations_filename, labelname):
        print("Loading the Wulczyn et al data ...")
        df = pd.read_csv(comments_filename, encoding = "utf-8", sep = '\t') 
        df_labs = pd.read_csv(annotations_filename, encoding = "utf-8", sep = '\t') 
        df_labs_aggregated = pd.DataFrame(df_labs.groupby(['rev_id'], as_index=False)[labelname].agg('mean'))
        
        df_joined = pd.merge(df, df_labs_aggregated, how='inner')
       
        df_joined["final_label"] = df_joined[labelname].apply(WulczynAnnotationMap)
        df_joined["comment"] = df_joined["comment"].apply(lambda s: s.replace("NEWLINE_TOKEN", " "))

        df_joined_train = df_joined[df_joined["split"] == "train"]
        df_joined_dev = df_joined[df_joined["split"] == "dev"]
        df_joined_test = df_joined[df_joined["split"] == "test"]

        self.train_texts = df_joined_train['comment']
        self.dev_texts = df_joined_dev['comment']
        self.test_texts = df_joined_test['comment']

        self.train_labels = df_joined_train['final_label']
        self.dev_labels = df_joined_dev['final_label']
        self.test_labels = df_joined_test['final_label']
        
        print("Finished loading!")



