from sklearn.svm import LinearSVC
from sklearn.model_selection import GridSearchCV
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import make_scorer, f1_score
import numpy as np
import scipy as sp

class my_model():
   
    def fit(self, train_X, train_y, dev_X, dev_y): # train with gridsearch to find hyperparameters included
        train_indices = np.array(range(len(train_y)))
        dev_indices = np.array(range(len(train_y), len(train_y) + len(dev_y)))

        custom_cv = [(train_indices, dev_indices)]
        
        my_scorer = make_scorer(f1_score, pos_label = 1, average = "binary")

        params = {"C": [2**i for i in range(-10,7)]}
        model = LinearSVC()

        clf = GridSearchCV(model, params, cv = custom_cv, verbose = 4, n_jobs = 16, scoring = my_scorer)
        
        
        train_dev_X = sp.sparse.vstack((train_X, dev_X))
        train_dev_y = train_y.append(dev_y)


        print("Fitting the model (including gridsearch) ...")
        clf.fit(train_dev_X, train_dev_y)
        self.clf = clf
    
    def predict(self, test_X):
        print("Making predictions on test data ...")
        return(self.clf.predict(test_X))



class my_model_feda():
   
    def feda_expand(self, X, current_domain_index, num_domains, domain_feat_counts):
        print("Starting feda expand ...")
        result = X[0]
        print("Initial shape:", result.shape)
        for i in range(num_domains):
            if i == current_domain_index:
                result = sp.sparse.hstack((result, X[1]))
            else:                
                zero_mat = sp.sparse.csr_matrix((X[0].shape[0], domain_feat_counts[i]))
                result = sp.sparse.hstack((result, zero_mat))       
            print("After iteration %d shape is %s" % (i, str(result.shape)))
        print("Finished feda expand")
        print(result.shape)
        return(result)
    
    def fit(self, train_X, train_y, dev_X, dev_y, dev_len): # train with gridsearch to find hyperparameters included

        train_indices = np.array(range(len(train_y)))
        dev_indices = np.array(range(len(train_y), len(train_y) + len(dev_y)))
        #dev_indices = np.array(range(len(train_y), len(train_y) + dev_len))

        custom_cv = [(train_indices, dev_indices)]
        
        my_scorer = make_scorer(f1_score, pos_label = 1, average = "binary")

        params = {"C": [2**i for i in range(-10,7)]}
        model = LinearSVC()

        clf = GridSearchCV(model, params, cv = custom_cv, verbose = 4, n_jobs = 16, scoring = my_scorer)
        
        train_dev_X = sp.sparse.vstack((train_X, dev_X))
        train_dev_y = train_y.append(dev_y)

        print("Fitting the model (including gridsearch) ...")
        clf.fit(train_dev_X, train_dev_y)
        self.clf = clf
    
    def predict(self, test_X_expanded):
        print("Making predictions on test data ...")
        return(self.clf.predict(test_X_expanded))


