import pandas as pd
from datareaders import *
import scipy as sp
from sklearn.feature_extraction.text import TfidfVectorizer
from mymodel import *
from sklearn.feature_selection import SelectKBest, chi2
import nltk

dataTRAC = TRACAggressionReader("./datasets/AggressionFB")
dataGO = GaoReader("./datasets/GaoEtAl")
dataKAGGLE = KaggleToxicJigsawReader("./datasets/KaggleToxicJigsaw")
dataKOL = KolhatkarReader("./datasets/Kolhatkar")
dataWas1 = WaseemNAACLReader("./datasets/WaseemHovyNAACL")
dataWas2 = WaseemNLPReader("./datasets/WaseemNLPaCSS")
dataWul1 = WulczynReader("./datasets/WulczynAggression/4267550/aggression_annotated_comments.tsv", "./datasets/WulczynAggression/4267550/aggression_annotations.tsv", "aggression")
dataWul2 = WulczynReader("./datasets/WulczynPersAttack/4054689/attack_annotated_comments.tsv", "./datasets/WulczynPersAttack/4054689/attack_annotations.tsv", "attack")
dataWul3 = WulczynReader("./datasets/WulczynToxicity/4563973/toxicity_annotated_comments.tsv", "./datasets/WulczynToxicity/4563973/toxicity_annotations.tsv", "toxicity")

datasets_list = [(dataTRAC, "TRAC"), (dataGO, "GO"), (dataKAGGLE, "KAGGLE"), (dataKOL, "KOL"), 
                (dataWas1, "Was1"), (dataWas2, "Was2"), (dataWul1, "Wul1"), (dataWul2, "Wul2"), 
                (dataWul3, "Wul3")]

#datasets_list = [(dataGO, "GO"), (dataKAGGLE, "KAGGLE")]

l = nltk.stem.WordNetLemmatizer()

def preprocessing(s):
	return l.lemmatize(s.lower())



for data_train, data_train_ID in datasets_list:
    print("******* STARTING FOR TRAIN " + data_train_ID) 
    print("Calculating feats ...")
    tiv = TfidfVectorizer(analyzer = "word", ngram_range = (1,1), norm = None, use_idf = False, stop_words = 'english', preprocessor = preprocessing)
    train_data = tiv.fit_transform(data_train.train_texts)
    dev_data = tiv.transform(data_train.dev_texts)
    m = my_model()
    m.fit(train_data, data_train.train_labels, dev_data, data_train.dev_labels)

    for data_test, data_test_ID in datasets_list:
        print("******* STARTING FOR TEST " + data_test_ID) 
        test_data = tiv.transform(data_test.test_texts)

        predictions  = pd.Series(m.predict(test_data))
        f1_classic = f1_score(y_true = data_test.test_labels, y_pred = predictions, pos_label = 1, average="binary")
        #print("TRUE --> " + str(len(data_test.test_labels)))
        #print("PRED --> " + str(len(predictions)))
        print("F1 classic :" + str(f1_classic))
        #print(predictions)
        output_frame = pd.DataFrame()
        output_frame["CORRECT"] = np.array(data_test.test_labels)
        output_frame["PREDICTION"] = np.array(predictions)
        #print(output_frame.shape)
        output_filename = "P1_" + data_train_ID + "_vs_" + data_test_ID + ".csv"
        output_frame.to_csv("./p1/"+output_filename)
        #if data_train_ID == "GO" and data_test_ID == "KAGGLE":
        #        exit()
  
