import pandas as pd
from datareaders import *
import scipy as sp
from sklearn.feature_extraction.text import TfidfVectorizer
import scipy as sp
from mymodel import *
import nltk

dataTRAC = TRACAggressionReader("./datasets/AggressionFB")
dataGO = GaoReader("./datasets/GaoEtAl")
dataKAGGLE = KaggleToxicJigsawReader("./datasets/KaggleToxicJigsaw")
dataKOL = KolhatkarReader("./datasets/Kolhatkar")
dataWas1 = WaseemNAACLReader("./datasets/WaseemHovyNAACL")
dataWas2 = WaseemNLPReader("./datasets/WaseemNLPaCSS")
dataWul1 = WulczynReader("./datasets/WulczynAggression/4267550/aggression_annotated_comments.tsv", "./datasets/WulczynAggression/4267550/aggression_annotations.tsv", "aggression")
dataWul2 = WulczynReader("./datasets/WulczynPersAttack/4054689/attack_annotated_comments.tsv", "./datasets/WulczynPersAttack/4054689/attack_annotations.tsv", "attack")
dataWul3 = WulczynReader("./datasets/WulczynToxicity/4563973/toxicity_annotated_comments.tsv", "./datasets/WulczynToxicity/4563973/toxicity_annotations.tsv", "toxicity")

datasets_list = [(dataTRAC, "TRAC"), (dataGO, "GO"), (dataKAGGLE, "KAGGLE"), (dataKOL, "KOL"), 
                (dataWas1, "Was1"), (dataWas2, "Was2"), (dataWul1, "Wul1"), (dataWul2, "Wul2"), 
                (dataWul3, "Wul3")]

augmentation_sets_list = [(dataTRAC, "TRAC"), (dataGO, "GO"), (dataKAGGLE, "KAGGLE"), (dataKOL, "KOL"), 
                (dataWas1, "Was1"), (dataWas2, "Was2"), (dataWul1, "Wul1"), (dataWul2, "Wul2"), 
                (dataWul3, "Wul3")]

l = nltk.stem.WordNetLemmatizer()

def preprocessing(s):
        return l.lemmatize(s.lower())



for orig_data, orig_data_ID in datasets_list:
    for aug_data, aug_data_ID in augmentation_sets_list:

        m = my_model_feda()

        num_domains = 2

        train_X_input = [orig_data.train_texts, aug_data.train_texts]
        train_y_input = [orig_data.train_labels, aug_data.train_labels]

        dev_X_input = [orig_data.dev_texts, aug_data.dev_texts]
        dev_y_input = [orig_data.dev_labels, aug_data.dev_labels]

        test_X_input = [orig_data.test_texts, aug_data.test_texts]
        test_y_input = [orig_data.test_labels, aug_data.test_labels]


        # ***************
        print("Fitting the common vectorizer ...")
        common_vectorizer =  TfidfVectorizer(analyzer = "word", ngram_range = (1,1), norm = None, use_idf = False, stop_words = 'english', preprocessor = preprocessing)
        common_vectorizer.fit(pd.concat(train_X_input))

        X_train_original_list, X_train_domain_list = [], []
        X_dev_original_list, X_dev_domain_list = [], []
        X_test_original_list, X_test_domain_list = [], []
        domain_feat_counts = []

        for i in range(num_domains):
            print("Calculating common feats for domain " + str(i))
            X_train_original_list.append(common_vectorizer.transform(train_X_input[i]))
            X_dev_original_list.append(common_vectorizer.transform(dev_X_input[i]))
            X_test_original_list.append(common_vectorizer.transform(test_X_input[i]))
            
            print("Fitting domain specific vectorizer for domain " + str(i))
            domain_specific_vectorizer = TfidfVectorizer(analyzer = "word", ngram_range = (1,1), norm = None, use_idf = False, stop_words = 'english', preprocessor = preprocessing)
            domain_specific_vectorizer.fit(train_X_input[i])

            print("Calculating domain specific feats for domain " + str(i))
            X_train_domain_list.append(domain_specific_vectorizer.transform(train_X_input[i]))
            X_dev_domain_list.append(domain_specific_vectorizer.transform(dev_X_input[i]))
            X_test_domain_list.append(domain_specific_vectorizer.transform(test_X_input[i]))

            domain_feat_counts.append(X_train_domain_list[-1].shape[1])


        train_X_expanded = [m.feda_expand(x, ind, num_domains, domain_feat_counts) for ind, x in enumerate(zip(X_train_original_list, X_train_domain_list))]
        dev_X_expanded = [m.feda_expand(x, ind, num_domains, domain_feat_counts) for ind, x in enumerate(zip(X_dev_original_list, X_dev_domain_list))]

        tr_X = sp.sparse.vstack(tuple(train_X_expanded))
        dv_X = sp.sparse.vstack(tuple(dev_X_expanded))

        tr_y = pd.concat(train_y_input)
        dv_y = pd.concat(dev_y_input)

        m.fit(tr_X, tr_y, dv_X, dv_y, len(orig_data.dev_labels))

        test_domain_index = 0 # domain to which we consider the test set belongs
        test_X_expanded = m.feda_expand((X_test_original_list[test_domain_index], X_test_domain_list[test_domain_index]), test_domain_index, num_domains, domain_feat_counts)

        predictions  = pd.Series(m.predict(test_X_expanded))

        f1_feda = f1_score(y_true = orig_data.test_labels, y_pred = predictions, pos_label = 1, average="binary")


        print("F1 score on test:")
        print("F1 feda " + str(f1_feda))
 
        output_frame = pd.DataFrame()
        output_frame["CORRECT"] = np.array(orig_data.test_labels)
        output_frame["PREDICTION"] = np.array(predictions)
        output_filename = "P2_" + orig_data_ID + "_vs_" + aug_data_ID + ".csv"
        output_frame.to_csv("./p2/"+output_filename, header = False) 
        



