import pandas as pd
from datareaders import *
import scipy as sp
from sklearn.feature_extraction.text import TfidfVectorizer
import scipy as sp
from mymodel import *
import nltk

#dataTRAC = TRACAggressionReader("./datasets/AggressionFB")
#dataGO = GaoReader("./datasets/GaoEtAl")
#dataKAGGLE = KaggleToxicJigsawReader("./datasets/KaggleToxicJigsaw")
#dataKOL = KolhatkarReader("./datasets/Kolhatkar")
#dataWas1 = WaseemNAACLReader("./datasets/WaseemHovyNAACL")
dataWas2 = WaseemNLPReader("./datasets/WaseemNLPaCSS")
#dataWul1 = WulczynReader("./datasets/WulczynAggression/4267550/aggression_annotated_comments.tsv", "./datasets/WulczynAggression/4267550/aggression_annotations.tsv", "aggression")
dataWul2 = WulczynReader("./datasets/WulczynPersAttack/4054689/attack_annotated_comments.tsv", "./datasets/WulczynPersAttack/4054689/attack_annotations.tsv", "attack")
#dataWul3 = WulczynReader("./datasets/WulczynToxicity/4563973/toxicity_annotated_comments.tsv", "./datasets/WulczynToxicity/4563973/toxicity_annotations.tsv", "toxicity")

datasets_list = [(dataWas2, "Was2", dataWul2, "Wul2")]

l = nltk.stem.WordNetLemmatizer()

def preprocessing(s):
        return l.lemmatize(s.lower())


for orig_data, orig_data_ID, aug_data, aug_data_ID in datasets_list:
        m = my_model_feda()

        num_domains = 2

        train_X_input = [orig_data.train_texts, aug_data.train_texts]
        train_y_input = [orig_data.train_labels, aug_data.train_labels]

        dev_X_input = [orig_data.dev_texts, aug_data.dev_texts]
        dev_y_input = [orig_data.dev_labels, aug_data.dev_labels]

        test_X_input = [orig_data.test_texts, aug_data.test_texts]
        test_y_input = [orig_data.test_labels, aug_data.test_labels]


        # ***************
        print("Fitting the common vectorizer ...")
        common_vectorizer =  TfidfVectorizer(analyzer = "word", ngram_range = (1,1), norm = None, use_idf = False, stop_words = 'english', preprocessor = preprocessing)
        common_vectorizer.fit(pd.concat(train_X_input))
        original_inv_vocab = {v: k for k, v in common_vectorizer.vocabulary_.items()}

        X_train_original_list, X_train_domain_list = [], []
        X_dev_original_list, X_dev_domain_list = [], []
        X_test_original_list, X_test_domain_list = [], []
        domain_feat_counts = []
        domain_inv_vocabs = []

        for i in range(num_domains):
            print("Calculating common feats for domain " + str(i))
            X_train_original_list.append(common_vectorizer.transform(train_X_input[i]))
            X_dev_original_list.append(common_vectorizer.transform(dev_X_input[i]))
            X_test_original_list.append(common_vectorizer.transform(test_X_input[i]))
            
            print("Fitting domain specific vectorizer for domain " + str(i))
            domain_specific_vectorizer = TfidfVectorizer(analyzer = "word", ngram_range = (1,1), norm = None, use_idf = False, stop_words = 'english', preprocessor = preprocessing)
            domain_specific_vectorizer.fit(train_X_input[i])

            print("Calculating domain specific feats for domain " + str(i))
            X_train_domain_list.append(domain_specific_vectorizer.transform(train_X_input[i]))
            X_dev_domain_list.append(domain_specific_vectorizer.transform(dev_X_input[i]))
            X_test_domain_list.append(domain_specific_vectorizer.transform(test_X_input[i]))

            domain_feat_counts.append(X_train_domain_list[-1].shape[1])
            domain_inv_vocab = {v: k for k, v in domain_specific_vectorizer.vocabulary_.items()}
            domain_inv_vocabs.append(domain_inv_vocab)
    
        train_X_expanded = [m.feda_expand(x, ind, num_domains, domain_feat_counts) for ind, x in enumerate(zip(X_train_original_list, X_train_domain_list))]
        dev_X_expanded = [m.feda_expand(x, ind, num_domains, domain_feat_counts) for ind, x in enumerate(zip(X_dev_original_list, X_dev_domain_list))]

        tr_X = sp.sparse.vstack(tuple(train_X_expanded))
        dv_X = sp.sparse.vstack(tuple(dev_X_expanded))

        tr_y = pd.concat(train_y_input)
        dv_y = pd.concat(dev_y_input)

        m.fit(tr_X, tr_y, dv_X, dv_y, len(orig_data.dev_labels))

        # reverse engineer where the features are

        coefs = m.clf.best_estimator_.coef_[0]
        orig_feat_weights = [(original_inv_vocab[i], coefs[i]) for i in range(len(original_inv_vocab))]
        orig_feat_weights_sorted = sorted(orig_feat_weights, key=lambda x: x[1], reverse=True)
        print("ORIGINAL FEATS:")
        print(orig_feat_weights_sorted[0:40])
        print(len(coefs))
        coefs = coefs[len(original_inv_vocab):]
        for cur_domain in range(num_domains):
                print(len(coefs))
                feat_weights = [(domain_inv_vocabs[cur_domain][i], coefs[i]) for i in range(len(domain_inv_vocabs[cur_domain]))]
                feat_weights_sorted = sorted(feat_weights, key=lambda x: x[1], reverse=True)
                print("FEATS for domain %d:" % cur_domain)
                print(feat_weights_sorted[0:40])
                if cur_domain < num_domains - 1:
                        coefs = coefs[len(domain_inv_vocabs[cur_domain]):]

        test_domain_index = 0 # domain to which we consider the test set belongs
        test_X_expanded = m.feda_expand((X_test_original_list[test_domain_index], X_test_domain_list[test_domain_index]), test_domain_index, num_domains, domain_feat_counts)

        predictions  = pd.Series(m.predict(test_X_expanded))

        f1_feda = f1_score(y_true = orig_data.test_labels, y_pred = predictions, pos_label = 1, average="binary")


        print("F1 score on test:")
        print("F1 feda " + str(f1_feda))
 
        output_frame = pd.DataFrame()
        output_frame["CORRECT"] = np.array(orig_data.test_labels)
        output_frame["PREDICTION"] = np.array(predictions)
        


